/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem, SMouseEvent } from "@sybotan-web/graphy/lib";
import { SColor, SPainter, SPoint, SRect } from "@sybotan-web/draw/lib";
import { Space } from "../types/Space";
import { Opt } from "../types/Opt";

/**
 * 空间item
 *
 * @author  郝建龙
 */
export class SpaceItem extends SGraphyItem {
    /** 空间所有数据  */
    data: Space;
    /** 空间轮廓线坐标list  */
    private readonly pointArr: SPoint[];

    /**
     * 构造函数
     *
     * @param data
     */
    constructor(data: Space) {
        super();
        this.data = data;
        this.pointArr = this.data.OutLine[0].map(t => {
            return new SPoint(t.X, t.Y);
        });
    } // Constructor

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        let pointArr = this.data.OutLine[0];
        let minX = pointArr[0].X;
        let maxX = minX;
        let minY = pointArr[0].Y;
        let maxY = minY;
        for (let i = 1; i < pointArr.length; i++) {
            if (pointArr[i].X < minX) {
                minX = pointArr[i].Y;
            }
            if (pointArr[i].Y < minY) {
                minY = pointArr[i].Y;
            }
            if (pointArr[i].X > maxX) {
                maxX = pointArr[i].X;
            }
            if (pointArr[i].Y > maxY) {
                maxY = pointArr[i].Y;
            }
        }

        return new SRect(minX, minY, maxX - minX, maxY - minY);
    } // Function boundingRect()

    /**
     * 判断点是否在区域内
     *
     * @param x
     * @param y
     */
    contains(x: number, y: number): boolean {
        let nCross = 0,
            point = [x, y],
            APoints = this.data.OutLine[0],
            length = APoints.length,
            p1,
            p2,
            i,
            xinters;
        p1 = APoints[0];
        for (i = 1; i <= length; i++) {
            p2 = APoints[i % length];
            if (
                point[0] > Math.min(p1.X, p2.X) &&
                point[0] <= Math.max(p1.X, p2.X)
            ) {
                if (point[1] <= Math.max(p1.Y, p2.Y)) {
                    if (p1.X != p2.X) {
                        //计算位置信息
                        xinters =
                            ((point[0] - p1.X) * (p2.Y - p1.Y)) /
                                (p2.X - p1.X) +
                            p1.Y;
                        if (p1.Y == p2.Y || point[1] <= xinters) {
                            nCross++;
                        }
                    }
                }
            }
            p1 = p2;
        }
        return nCross % 2 === 1;
    } // Function contains()

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     * @param   rect          绘制区域
     */
    onDraw(painter: SPainter, rect?: SRect): void {
        if (this.visible) {
            painter.pen.color = SColor.Transparent;
            painter.brush.color = Opt.spaceColor;
            painter.pen.lineWidth = 200;
            painter.drawPolygon(this.pointArr);
        }
    } // Function onDraw()
} // Class SpaceItem
